<?php
/**
 * Plugin Name:  BuddyPress Bump Activity to Top on Comment
 * Version:      1.0.0
 * Plugin URI:   https://buddydev.com/support/forums/
 * Description:  This plugin moves activity to top of the stream when a new comment is received.
 * Author:       BuddyDev
 * Author URI:   https://buddydev.com/
 * Requires PHP: 5.3
 * License:      GPL2
 * License URI:  https://www.gnu.org/licenses/gpl-2.0.html
 **/

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

if ( class_exists( 'BuddyDev_Bump_Activity_On_Comment' ) ) {
	return;
}

/**
 * Helper class for Bumping activity to top on new comment.
 */
class BuddyDev_Bump_Activity_On_Comment {

	/**
	 * Singleton instance.
	 *
	 * @var BuddyDev_Bump_Activity_On_Comment
	 */
	private static $instance = null;

	/**
	 * Singleton factory method.
	 *
	 * @return BuddyDev_Bump_Activity_On_Comment|null
	 */
	public static function instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
			self::$instance->setup();
		}

		return self::$instance;
	}

	/**
	 * Constructor.
	 */
	private function __construct() {
	}

	/**
	 * Setup.
	 */
	private function setup() {

		add_action( 'bp_activity_comment_posted', array( $this, 'update_activity_time_on_comment' ), 10, 3 );

		// Filter recorded date for activity.
		add_filter( 'bp_get_activity_date_recorded', array( $this, 'filter_activity_date_recorded' ) );

		// filter time since for the activity.
		add_filter( 'bp_activity_time_since', array( $this, 'filter_activity_time_since' ), 100, 2 );
	}

	/**
	 * Update time on new activity comment.
	 *
	 * @param int                  $comment_id comment id.
	 * @param array                $r args.
	 * @param BP_Activity_Activity $activity activity object.
	 */
	public function update_activity_time_on_comment( $comment_id, $r, $activity ) {

		$created_at = bp_activity_get_meta( $activity->id, '_bua_activity_created_at', true );

		if ( ! $created_at ) {
			$created_at = $activity->date_recorded;
		}

		// modify.
		$activity->date_recorded = gmdate( 'Y-m-d H:i:s' );
		$activity->save();
		// Keep original activity creation time.
		bp_activity_update_meta( $activity->id, '_bua_activity_created_at', $created_at );
	}

	/**
	 * Filters recorded date.
	 *
	 * @param string $date record date.
	 *
	 * @return string
	 */
	public function filter_activity_date_recorded( $date ) {

		global $activities_template;

		$activity = $activities_template->activity;

		$created_at = bp_activity_get_meta( $activity->id, '_bua_activity_created_at', true );

		if ( ! empty( $created_at ) ) {
			$date = $created_at;
		}

		return $date;
	}

	/**
	 * Filters activity time since.
	 *
	 * @param string               $time_since time since string.
	 * @param BP_Activity_Activity $activity activity object.
	 *
	 * @return string
	 */
	public function filter_activity_time_since( $time_since, $activity ) {

		$created_at = bp_activity_get_meta( $activity->id, '_bua_activity_created_at', true );

		if ( empty( $created_at ) ) {
			return $time_since;
		}

		$time_since = sprintf(
			'<span class="time-since" data-livestamp="%1$s">%2$s</span>',
			bp_core_get_iso8601_date( $created_at ),
			bp_core_time_since( $created_at )
		);

		return $time_since;
	}
}

// Instantiate.
BuddyDev_Bump_Activity_On_Comment::instance();
